<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use League\OAuth2\Server\AuthorizationServer;

class OAuth2Controller extends AbstractController
{
    /**
     * This controller intercepts the /authorize route before the League OAuth2 Server
     * bundle handles it. If the user is logged in, it will pass through to the OAuth2
     * server. If not, it will redirect to the login page.
     */
    #[Route('/authorize', name: 'oauth2_authorize_check', methods: ['GET', 'POST'])]
    public function authorize(Request $request): Response
    {
        // If the user is not logged in, redirect to login
        if (!$this->getUser()) {
            // Store the original request parameters
            $session = $request->getSession();
            $session->set('oauth_authorization_request', $request->query->all());
            
            // Redirect to login
            return $this->redirectToRoute('app_login');
        }
        
        // User is logged in, let the OAuth2 server handle the authorization
        // We'll just forward the request to the League OAuth2 Server bundle's controller
        return $this->forward('league.oauth2_server.controller.authorization::indexAction');
    }
} 