<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Security\Http\Authentication\AuthenticationUtils;
use KnpU\OAuth2ClientBundle\Client\ClientRegistry;

final class SecurityController extends AbstractController
{
    #[Route('/login', name: 'app_login')]
    public function login(AuthenticationUtils $authenticationUtils, Request $request): Response
    {
        // get the login error if there is one
        $error = $authenticationUtils->getLastAuthenticationError();
        
        // last username entered by the user
        $lastUsername = $authenticationUtils->getLastUsername();

        return $this->render('security/login.html.twig', [
            'last_username' => $lastUsername,
            'error' => $error,
        ]);
    }

    #[Route('/logout', name: 'app_logout')]
    public function logout(): void
    {
        // This method can be empty - it will be intercepted by the logout key on your firewall
        // The logout is handled by Symfony's security system
    }

    #[Route('/', name: 'app_home')]
    public function home(): Response
    {
        return $this->render('security/home.html.twig', [
            'user' => $this->getUser(),
        ]);
    }

    #[Route('/connect/sso', name: 'connect_sso_start')]
    public function connectAction(ClientRegistry $clientRegistry): Response
    {
        return $clientRegistry
            ->getClient('sso_server')
            ->redirect([
                'profile',
                'email'
            ]);
    }

    #[Route('/connect/sso/check', name: 'connect_sso_check')]
    public function connectCheckAction(): Response
    {
        // This method can be empty - it will be intercepted by the oauth2 firewall
        throw new \Exception('Don\'t forget to activate check_path in security.yaml');
    }
    
    /**
     * This route is used after successful authentication to redirect
     * back to the OAuth2 authorization if there was a pending request
     */
    #[Route('/login/check', name: 'app_login_check')]
    public function loginCheck(Request $request): Response
    {
        // Check if there was a pending OAuth2 authorization request
        $session = $request->getSession();
        $oauthRequest = $session->get('oauth_authorization_request');
        
        if ($oauthRequest) {
            // Clear the stored request
            $session->remove('oauth_authorization_request');
            
            // Rebuild the authorization URL
            $queryString = http_build_query($oauthRequest);
            return $this->redirect('/authorize?' . $queryString);
        }
        
        // No pending OAuth2 request, go to homepage
        return $this->redirectToRoute('app_home');
    }
}
