<?php

namespace App\Entity;

use App\Repository\AccessTokenRepository;
use Doctrine\ORM\Mapping as ORM;
use League\Bundle\OAuth2ServerBundle\Model\AccessTokenInterface;
use League\Bundle\OAuth2ServerBundle\Model\ClientInterface;
use League\Bundle\OAuth2ServerBundle\Model\Scope;
use League\Bundle\OAuth2ServerBundle\ValueObject\Scope as ScopeVO;

#[ORM\Entity(repositoryClass: AccessTokenRepository::class)]
class AccessToken implements AccessTokenInterface
{
    #[ORM\Id]
    #[ORM\Column(length: 80)]
    private string $identifier;

    #[ORM\Column]
    private \DateTimeImmutable $expiry;

    #[ORM\Column]
    private string $userIdentifier;

    #[ORM\Column]
    private string $clientIdentifier;

    #[ORM\Column]
    private bool $revoked = false;

    #[ORM\Column]
    private array $scopes = [];

    public function __construct(
        string $identifier,
        \DateTimeImmutable $expiry,
        string $userIdentifier,
        string $clientIdentifier
    ) {
        $this->identifier = $identifier;
        $this->expiry = $expiry;
        $this->userIdentifier = $userIdentifier;
        $this->clientIdentifier = $clientIdentifier;
    }

    public function getIdentifier(): string
    {
        return $this->identifier;
    }

    public function getExpiry(): \DateTimeImmutable
    {
        return $this->expiry;
    }
    
    /**
     * Proxies to getExpiry for backward compatibility with older versions of the library
     */
    public function getExpiryDateTime(): \DateTimeImmutable
    {
        return $this->getExpiry();
    }

    public function getUserIdentifier(): string
    {
        return $this->userIdentifier;
    }

    public function getClient(): ClientInterface
    {
        throw new \RuntimeException('Method not implemented.');
    }

    public function getClientIdentifier(): string
    {
        return $this->clientIdentifier;
    }

    public function isRevoked(): bool
    {
        return $this->revoked;
    }

    public function revoke(): AccessTokenInterface
    {
        $this->revoked = true;
        return $this;
    }

    public function getScopes(): array
    {
        return array_map(
            static fn (string $scope): Scope => new ScopeVO($scope),
            $this->scopes
        );
    }

    public function setScopes(array $scopes): self
    {
        $this->scopes = array_map(
            static fn (Scope $scope): string => (string) $scope,
            $scopes
        );
        return $this;
    }

    public function __toString(): string
    {
        return $this->getIdentifier();
    }
} 