<?php

namespace App\Entity;

use App\Repository\RefreshTokenRepository;
use Doctrine\ORM\Mapping as ORM;
use League\Bundle\OAuth2ServerBundle\Model\AccessTokenInterface;
use League\Bundle\OAuth2ServerBundle\Model\RefreshTokenInterface;

#[ORM\Entity(repositoryClass: RefreshTokenRepository::class)]
class RefreshToken implements RefreshTokenInterface
{
    #[ORM\Id]
    #[ORM\Column(length: 80)]
    private string $identifier;

    #[ORM\Column]
    private \DateTimeImmutable $expiry;

    #[ORM\Column(length: 80)]
    private string $accessTokenIdentifier;

    #[ORM\Column]
    private bool $revoked = false;

    public function __construct(
        string $identifier,
        \DateTimeImmutable $expiry,
        string $accessTokenIdentifier
    ) {
        $this->identifier = $identifier;
        $this->expiry = $expiry;
        $this->accessTokenIdentifier = $accessTokenIdentifier;
    }

    public function getIdentifier(): string
    {
        return $this->identifier;
    }

    public function getExpiry(): \DateTimeImmutable
    {
        return $this->expiry;
    }
    
    /**
     * Proxies to getExpiry for backward compatibility with older versions of the library
     */
    public function getExpiryDateTime(): \DateTimeImmutable
    {
        return $this->getExpiry();
    }

    public function getAccessToken(): AccessTokenInterface
    {
        throw new \RuntimeException('Method not implemented.');
    }

    public function getAccessTokenIdentifier(): string
    {
        return $this->accessTokenIdentifier;
    }

    public function isRevoked(): bool
    {
        return $this->revoked;
    }

    public function revoke(): RefreshTokenInterface
    {
        $this->revoked = true;
        return $this;
    }

    public function __toString(): string
    {
        return $this->getIdentifier();
    }
} 