<?php

namespace App\Repository;

use App\Entity\AccessToken;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use League\Bundle\OAuth2ServerBundle\Model\AccessTokenInterface;
use League\Bundle\OAuth2ServerBundle\Model\ClientInterface;
use League\Bundle\OAuth2ServerBundle\Repository\AccessTokenRepositoryInterface;
use League\Bundle\OAuth2ServerBundle\ValueObject\Scope;

/**
 * @extends ServiceEntityRepository<AccessToken>
 */
class AccessTokenRepository extends ServiceEntityRepository implements AccessTokenRepositoryInterface
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, AccessToken::class);
    }

    public function save(AccessToken $entity, bool $flush = false): void
    {
        $this->getEntityManager()->persist($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function remove(AccessToken $entity, bool $flush = false): void
    {
        $this->getEntityManager()->remove($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function getAccessToken(string $identifier): ?AccessTokenInterface
    {
        return $this->find($identifier);
    }

    public function persistNewAccessToken(AccessTokenInterface $accessToken): void
    {
        if (!$accessToken instanceof AccessToken) {
            throw new \RuntimeException('Invalid access token type');
        }

        $this->save($accessToken, true);
    }

    public function revokeAccessToken(string $identifier): void
    {
        $accessToken = $this->find($identifier);
        if (null !== $accessToken) {
            $accessToken->revoke();
            $this->save($accessToken, true);
        }
    }

    public function isAccessTokenRevoked(string $identifier): bool
    {
        $accessToken = $this->find($identifier);
        if (null === $accessToken) {
            return true;
        }

        return $accessToken->isRevoked();
    }

    public function createNewAccessToken(
        ClientInterface $client,
        array $scopes,
        string $userIdentifier = null
    ): AccessTokenInterface {
        $accessToken = new AccessToken(
            bin2hex(random_bytes(40)),
            new \DateTimeImmutable('+1 hour'),
            $userIdentifier ?? '',
            $client->getIdentifier()
        );

        $accessToken->setScopes(array_map(
            static fn(string $scope) => new Scope($scope),
            array_map(
                static fn($scope) => (string) $scope,
                $scopes
            )
        ));

        return $accessToken;
    }
} 