<?php

namespace App\Repository;

use App\Entity\AuthCode;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use League\Bundle\OAuth2ServerBundle\Model\AuthorizationCodeInterface;
use League\Bundle\OAuth2ServerBundle\Model\ClientInterface;
use League\Bundle\OAuth2ServerBundle\Repository\AuthorizationCodeRepositoryInterface;
use League\Bundle\OAuth2ServerBundle\ValueObject\Scope;

/**
 * @extends ServiceEntityRepository<AuthCode>
 */
class AuthCodeRepository extends ServiceEntityRepository implements AuthorizationCodeRepositoryInterface
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, AuthCode::class);
    }

    public function save(AuthCode $entity, bool $flush = false): void
    {
        $this->getEntityManager()->persist($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function remove(AuthCode $entity, bool $flush = false): void
    {
        $this->getEntityManager()->remove($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function getAuthorizationCode(string $identifier): ?AuthorizationCodeInterface
    {
        return $this->find($identifier);
    }

    public function persistNewAuthorizationCode(AuthorizationCodeInterface $authorizationCode): void
    {
        if (!$authorizationCode instanceof AuthCode) {
            throw new \RuntimeException('Invalid authorization code type');
        }

        $this->save($authorizationCode, true);
    }

    public function revokeAuthorizationCode(string $identifier): void
    {
        $authCode = $this->find($identifier);
        if (null !== $authCode) {
            $authCode->revoke();
            $this->save($authCode, true);
        }
    }

    public function isAuthorizationCodeRevoked(string $identifier): bool
    {
        $authCode = $this->find($identifier);
        if (null === $authCode) {
            return true;
        }

        return $authCode->isRevoked();
    }

    public function createNewAuthorizationCode(
        ClientInterface $client,
        array $scopes,
        ?string $userIdentifier,
        ?string $redirectUri
    ): AuthorizationCodeInterface {
        $authCode = new AuthCode(
            bin2hex(random_bytes(40)),
            new \DateTimeImmutable('+5 minutes'),
            $userIdentifier ?? '',
            $client->getIdentifier(),
            $redirectUri
        );

        $authCode->setScopes(array_map(
            static fn(string $scope) => new Scope($scope),
            array_map(
                static fn($scope) => (string) $scope,
                $scopes
            )
        ));

        return $authCode;
    }
} 