<?php

namespace App\Repository;

use App\Entity\Client;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\OAuth2\Server\Repositories\ClientRepositoryInterface;

/**
 * @extends ServiceEntityRepository<Client>
 */
class ClientRepository extends ServiceEntityRepository implements ClientRepositoryInterface
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Client::class);
    }

    public function save(Client $entity, bool $flush = false): void
    {
        $this->getEntityManager()->persist($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function remove(Client $entity, bool $flush = false): void
    {
        $this->getEntityManager()->remove($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function getClientEntity(string $clientIdentifier): ?ClientEntityInterface
    {
        return $this->find($clientIdentifier);
    }

    public function validateClient(string $clientIdentifier, ?string $clientSecret, ?string $grantType): bool
    {
        $client = $this->find($clientIdentifier);

        if (null === $client || !$client->isActive()) {
            return false;
        }

        // Validate grant type
        if (null !== $grantType) {
            $validGrantTypes = array_map(
                static fn($grant) => (string) $grant,
                $client->getGrants()
            );

            if (!in_array($grantType, $validGrantTypes, true)) {
                return false;
            }
        }

        // Validate secret
        if (null === $client->getSecret()) {
            return true;
        }

        if (null === $clientSecret) {
            return false;
        }

        return $client->getSecret() === $clientSecret;
    }
} 