<?php

namespace App\Repository;

use App\Entity\RefreshToken;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use League\Bundle\OAuth2ServerBundle\Model\AccessTokenInterface;
use League\Bundle\OAuth2ServerBundle\Model\RefreshTokenInterface;
use League\Bundle\OAuth2ServerBundle\Repository\RefreshTokenRepositoryInterface;

/**
 * @extends ServiceEntityRepository<RefreshToken>
 */
class RefreshTokenRepository extends ServiceEntityRepository implements RefreshTokenRepositoryInterface
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, RefreshToken::class);
    }

    public function save(RefreshToken $entity, bool $flush = false): void
    {
        $this->getEntityManager()->persist($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function remove(RefreshToken $entity, bool $flush = false): void
    {
        $this->getEntityManager()->remove($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function getRefreshToken(string $identifier): ?RefreshTokenInterface
    {
        return $this->find($identifier);
    }

    public function persistNewRefreshToken(RefreshTokenInterface $refreshToken): void
    {
        if (!$refreshToken instanceof RefreshToken) {
            throw new \RuntimeException('Invalid refresh token type');
        }

        $this->save($refreshToken, true);
    }

    public function revokeRefreshToken(string $identifier): void
    {
        $refreshToken = $this->find($identifier);
        if (null !== $refreshToken) {
            $refreshToken->revoke();
            $this->save($refreshToken, true);
        }
    }

    public function isRefreshTokenRevoked(string $identifier): bool
    {
        $refreshToken = $this->find($identifier);
        if (null === $refreshToken) {
            return true;
        }

        return $refreshToken->isRevoked();
    }

    public function createNewRefreshToken(
        AccessTokenInterface $accessToken,
        \DateTimeImmutable $expiry
    ): RefreshTokenInterface {
        return new RefreshToken(
            bin2hex(random_bytes(40)),
            $expiry,
            $accessToken->getIdentifier()
        );
    }
} 